﻿Imports System.Net
Imports System.IO
Imports System.Text

Public Class ApiContext
#Region "--- Fields ---"
    Private _api_key As Guid = Guid.Empty
    Private _news As List(Of newsItem)
    Private _apiRootUrl As String
    Private _apiRootXml As XmlNode
    Private _newsurl As String
    Private _baseurl As String
#End Region

    ''' <summary>
    ''' Api_Key and BaseUrl properties much be set
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        'Me.api_key = SampleClientLibraryConfigurationSection.Config.ApiKey
    End Sub

    Public Sub New(ByVal api_key As String, ByVal BaseUrl As String)
        Me.New(New Guid(api_key), BaseUrl)
    End Sub

    Public Sub New(ByVal api_key As Guid, ByVal BaseUrl As String)
        Me.Api_Key = api_key
        _baseurl = BaseUrl
    End Sub

    Public Property BaseUrl() As String
        Get
            Return _baseurl
        End Get
        Set(ByVal value As String)
            _baseurl = value
        End Set
    End Property

    Public Property Api_Key() As Guid
        Get
            Return _api_key
        End Get
        Set(ByVal value As Guid)
            _api_key = value
        End Set
    End Property

    Friend ReadOnly Property RootUrl() As String
        Get
            Return Me.BaseUrl & Api_Key.ToString & "/"
        End Get
    End Property

    Friend ReadOnly Property NewsUrl() As String
        Get
            If String.IsNullOrEmpty(_newsurl) Then
                Try
                    _newsurl = ApiRootXml.Item("news").Attributes("href").Value
                Catch ex As Exception
                    Throw New Exception("Failed to parse the News URL", ex)
                End Try
            End If

            Return _newsurl
        End Get
    End Property

    Private ReadOnly Property ApiRootXml() As XmlNode
        Get
            If _apiRootXml Is Nothing Then
                If Not String.IsNullOrEmpty(Me.RootUrl) Then
                    Dim xmlDoc As New XmlDocument
                    xmlDoc.Load(Me.RootUrl)
                    _apiRootXml = xmlDoc.Item("api")
                Else
                    Throw New Exception("No API root URL provided!")
                End If
            End If
            Return _apiRootXml
        End Get
    End Property

    ''' <summary>
    ''' This readonly property presents a lazy-loaded enumeration of newsItems read,
    ''' when requested, from the restful api. This throws exceptions if the xml in
    ''' api is not able to be reached or the items are not able to be parsed.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    ''' <exception cref="Exceptions.ApiDecodeException"></exception>
    ''' <exception cref="Exceptions.ApiNotAvailableException"></exception>
    Public ReadOnly Property News() As IEnumerable(Of newsItem)
        Get
            If _news Is Nothing Then
                _news = New List(Of newsItem)
                'Load news from service
                Dim NewsXMLDoc As XmlDocument
                Try
                    If String.IsNullOrEmpty(Me.NewsUrl) Then
                        Throw New Exception("News URL is empty!")
                    Else
                        NewsXMLDoc = New XmlDocument
                        NewsXMLDoc.Load(Me.NewsUrl) '-- load the list of News Items
                    End If
                Catch ex As Exception
                    Throw New Exceptions.ApiNotAvailableException(ex)
                End Try
                Try
                    Dim NewsItemEnum As IEnumerator = NewsXMLDoc.Item("news").ChildNodes.GetEnumerator
                    While NewsItemEnum.MoveNext
                        Dim NewsListItem As XmlNode = DirectCast(NewsItemEnum.Current, XmlNode)
                        If NewsListItem.Name = "newsListItem" Then
                            Try
                                _news.Add(New newsItem(NewsListItem))
                            Catch ex As NullReferenceException
                                'don't throw an exception here if you want to try loading the next item.
                            End Try
                        End If
                    End While
                Catch ex As Exception
                    Throw New Exceptions.ApiDecodeException(ex)
                End Try
            End If
            Return _news
        End Get
    End Property

    Public Function postComment(ByVal articleID As Integer, ByVal name As String, ByVal location As String, ByVal text As String) As Boolean
        Dim postRequest As HttpWebRequest = HttpWebRequest.Create(NewsUrl & articleID & "/comments/post/")
        postRequest.Method = "POST"
        postRequest.ContentType = "text/xml"

        Dim doc As New XmlDocument()
        Dim commentElement As XmlElement = doc.CreateElement("comment")

        Dim idElement As XmlElement = doc.CreateElement("id")
        idElement.InnerText = "0"
        Dim publishedDateElement As XmlElement = doc.CreateElement("publishedDate")
        publishedDateElement.InnerText = Date.Now.ToString("s")
        Dim nameElement As XmlElement = doc.CreateElement("name")
        nameElement.InnerText = name
        Dim locationElement As XmlElement = doc.CreateElement("location")
        locationElement.InnerText = location
        Dim textElement As XmlElement = doc.CreateElement("text")
        textElement.InnerText = text

        commentElement.AppendChild(idElement)
        commentElement.AppendChild(publishedDateElement)
        commentElement.AppendChild(nameElement)
        commentElement.AppendChild(locationElement)
        commentElement.AppendChild(textElement)

        doc.AppendChild(commentElement)

        Dim postData As String = doc.OuterXml
        postRequest.ContentLength = postData.Length

        Try
            Dim requestStream As Stream = postRequest.GetRequestStream()
            requestStream.Write(Encoding.UTF8.GetBytes(postData.ToCharArray()), 0, postData.Length)
            requestStream.Close()

            Dim response As WebResponse = postRequest.GetResponse()
            Dim responseStatus = CType(response, HttpWebResponse).StatusDescription
            If responseStatus = "OK" Then
                Dim responseStream As Stream = response.GetResponseStream()
                Dim reader As StreamReader = New StreamReader(responseStream, Encoding.UTF8)
                Dim responseAsXML As XmlDocument = New XmlDocument()
                responseAsXML.LoadXml(reader.ReadToEnd())

                If responseAsXML.InnerText = "true" Then
                    Return True
                Else
                    Return False
                End If
            Else
                Return False
            End If
        Catch ex As Exception
            Return False
        End Try
    End Function
    'Public Function getNewsItem(ByVal articleID As Integer) As newsItem
    '    Dim newsItemUrl As String = Me.NewsUrl & articleID
    '    Dim NewsXMLDoc As XmlDocument
    '    Try
    '        If String.IsNullOrEmpty(Me.NewsUrl) Then
    '            Throw New Exception("News URL is empty!")
    '        Else
    '            NewsXMLDoc = New XmlDocument
    '            NewsXMLDoc.Load(newsItemUrl) '-- load the list of News Items
    '        End If
    '    Catch ex As Exception
    '        Throw New Exceptions.ApiNotAvailableException(ex)
    '    End Try
    '    Try
    '        Dim id As Integer
    '        Integer.TryParse(articleID, id)
    '        Dim headline As String = NewsXMLDoc.Item("newsItem").Item("headline").InnerText
    '        Dim publishDate As Date
    '        Date.TryParse(NewsXMLDoc.Item("newsItem").Item("publishDate").InnerText, publishDate)

    '        Return New newsItem(id, headline, publishDate, newsItemUrl)
    '    Catch ex As Exception
    '        Throw New Exceptions.ApiDecodeException
    '    End Try
    'End Function
End Class
